var __defProp = Object.defineProperty;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __objRest = (source, exclude) => {
  var target = {};
  for (var prop in source)
    if (__hasOwnProp.call(source, prop) && exclude.indexOf(prop) < 0)
      target[prop] = source[prop];
  if (source != null && __getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(source)) {
      if (exclude.indexOf(prop) < 0 && __propIsEnum.call(source, prop))
        target[prop] = source[prop];
    }
  return target;
};
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve2, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve2(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};

// src/generate/base.ts
import ts2 from "typescript";

// src/program.ts
import * as path from "path";
import ts from "typescript";
var cache = /* @__PURE__ */ new Map();
function getFileSymbol(file, program) {
  const checker = program.getTypeChecker();
  const sourceFile = program.getSourceFile(file);
  if (!sourceFile)
    return;
  return checker.getSymbolAtLocation(sourceFile);
}
function getProgram(options = {}) {
  var _a, _b, _c, _d;
  const key = JSON.stringify(options);
  const cached = cache.get(key);
  if (cached)
    return cached;
  const configFile = ts.readJsonConfigFile(
    (_a = options.tsconfigPath) != null ? _a : "./tsconfig.json",
    (p) => ts.sys.readFile(p)
  );
  const parsed = ts.parseJsonSourceFileConfigFileContent(
    configFile,
    ts.sys,
    (_b = options.basePath) != null ? _b : "./"
  );
  parsed.options.incremental = false;
  const host = ts.createCompilerHost(parsed.options);
  if (options.getDefaultLibLocation !== "default") {
    host.getDefaultLibLocation = (_c = options.getDefaultLibLocation) != null ? _c : () => path.resolve("./node_modules/typescript/lib");
  }
  const program = ts.createProgram({
    rootNames: (_d = options.files) != null ? _d : parsed.fileNames,
    host,
    options: parsed.options,
    configFileParsingDiagnostics: parsed.errors,
    projectReferences: parsed.projectReferences
  });
  cache.set(key, program);
  return program;
}

// src/generate/base.ts
function generateDocumentation(file, name, options = {}) {
  const program = getProgram(options.config);
  const fileSymbol = getFileSymbol(file, program);
  if (!fileSymbol)
    return;
  const symbol = program.getTypeChecker().getExportsOfModule(fileSymbol).find((e) => e.getEscapedName().toString() === name);
  if (!symbol)
    return;
  return generate(program, symbol, options);
}
function generate(program, symbol, { allowInternal = false, transform }) {
  const checker = program.getTypeChecker();
  const type = checker.getDeclaredTypeOfSymbol(symbol);
  const entryContext = {
    checker,
    transform,
    program,
    type,
    symbol
  };
  return {
    name: symbol.getEscapedName().toString(),
    description: ts2.displayPartsToString(
      symbol.getDocumentationComment(checker)
    ),
    entries: type.getProperties().map((prop) => getDocEntry(prop, entryContext)).filter((entry) => allowInternal || !("internal" in entry.tags))
  };
}
function getDocEntry(prop, context) {
  var _a, _b;
  const { checker, transform } = context;
  const subType = checker.getTypeOfSymbol(prop);
  const tags = Object.fromEntries(
    prop.getJsDocTags().map((tag) => [tag.name, ts2.displayPartsToString(tag.text)])
  );
  let typeName = checker.typeToString(
    subType.getNonNullableType(),
    void 0,
    ts2.TypeFormatFlags.UseAliasDefinedOutsideCurrentScope
  );
  if (subType.aliasSymbol && !subType.aliasTypeArguments) {
    typeName = subType.aliasSymbol.escapedName.toString();
  }
  if (tags.remarks) {
    typeName = (_b = (_a = new RegExp("^`(?<name>.+)`").exec(tags.remarks)) == null ? void 0 : _a[1]) != null ? _b : typeName;
  }
  const entry = {
    name: prop.getName(),
    description: ts2.displayPartsToString(prop.getDocumentationComment(checker)),
    tags,
    type: typeName
  };
  transform == null ? void 0 : transform.call(context, entry, subType, prop);
  return entry;
}

export {
  __spreadValues,
  __objRest,
  __async,
  getFileSymbol,
  getProgram,
  generateDocumentation,
  generate
};
